@@ -1,26 +1,14 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
 #include <Kokkos_Macros.hpp>
 #ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+import kokkos.core;
 import kokkos.std_algorithms;
 #else
+#include <Kokkos_Core.hpp>
 #include <Kokkos_StdAlgorithms.hpp>
 #endif
-#include <Kokkos_Core.hpp>
 #include <string>
 
 namespace Test {
@@ -514,7 +502,6 @@ struct TestStruct {
   }
 };
 
-#ifndef KOKKOS_ENABLE_CXX17
 template <typename ViewType>
 constexpr bool
 test_kokkos_iterator_satify_std_random_access_iterator_concept() {
@@ -526,7 +513,6 @@ static_assert(test_kokkos_iterator_satify_std_random_access_iterator_concept<
               Kokkos::View<int *>>());
 static_assert(test_kokkos_iterator_satify_std_random_access_iterator_concept<
               Kokkos::View<const int *>>());
-#endif
 
 }  // namespace compileonly
 }  // namespace stdalgos
