@@ -1,20 +1,12 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
+#include <Kokkos_Macros.hpp>
+#ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+import kokkos.core;
+#else
 #include <Kokkos_Core.hpp>
+#endif
 #include <gtest/gtest.h>
 
 // purpose of this test is to check that the reducers used
@@ -72,7 +64,7 @@ auto create_host_view_with_reduction_order_indices(
     result(8) = 7;
     result(9) = 5;
   } else {
-    throw std::runtime_error("test: Invalid enum");
+    Kokkos::abort("test: Invalid enum");
   }
 
   return result;
@@ -80,7 +72,7 @@ auto create_host_view_with_reduction_order_indices(
 
 template <int flag, class ExeSpace, class IndexType, class ViewType>
 auto run_min_or_max_test(ViewType view, StdReducersTestEnumOrder enValue) {
-  static_assert(std::is_same<ExeSpace, Kokkos::HostSpace>::value,
+  static_assert(std::is_same_v<ExeSpace, Kokkos::HostSpace>,
                 "test is only enabled for HostSpace");
 
   using view_value_type = typename ViewType::value_type;
@@ -191,7 +183,7 @@ template <class ExeSpace, class IndexType, class ViewType, class ValuesPair,
           class IndexPair>
 void run_min_max_test(ViewType view, StdReducersTestEnumOrder enValue,
                       const ValuesPair gold_values, const IndexPair gold_locs) {
-  static_assert(std::is_same<ExeSpace, Kokkos::HostSpace>::value,
+  static_assert(std::is_same_v<ExeSpace, Kokkos::HostSpace>,
                 "test is only enabled for HostSpace");
 
   using view_value_type = typename ViewType::value_type;
