@@ -1,18 +1,5 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
 namespace Test {
 template <class DeviceType, typename ScalarType = double,
@@ -24,7 +11,7 @@ struct MultiDimRangePerf3D {
   using iterate_type = Kokkos::Iterate;
 
   using view_type      = Kokkos::View<ScalarType ***, TestLayout, DeviceType>;
-  using host_view_type = typename view_type::HostMirror;
+  using host_view_type = typename view_type::host_mirror_type;
 
   view_type A;
   view_type B;
@@ -285,7 +272,7 @@ struct RangePolicyCollapseTwo {
   using iterate_type = Kokkos::Iterate;
 
   using view_type      = Kokkos::View<ScalarType ***, TestLayout, DeviceType>;
-  using host_view_type = typename view_type::HostMirror;
+  using host_view_type = typename view_type::host_mirror_type;
 
   view_type A;
   view_type B;
@@ -457,7 +444,7 @@ struct RangePolicyCollapseAll {
   using layout          = TestLayout;
 
   using view_type      = Kokkos::View<ScalarType ***, TestLayout, DeviceType>;
-  using host_view_type = typename view_type::HostMirror;
+  using host_view_type = typename view_type::host_mirror_type;
 
   view_type A;
   view_type B;
