@@ -1,18 +1,5 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
 #ifndef KOKKOS_OPENMPTARGET_PARALLELSCAN_RANGE_HPP
 #define KOKKOS_OPENMPTARGET_PARALLELSCAN_RANGE_HPP
@@ -60,7 +47,9 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
           chunk_values,
       Kokkos::View<int64_t, Kokkos::Experimental::OpenMPTargetSpace> count)
       const {
-    const idx_type N          = m_policy.end() - m_policy.begin();
+    const idx_type begin      = m_policy.begin();
+    const idx_type end        = m_policy.end();
+    const idx_type N          = end - begin;
     const idx_type chunk_size = 128;
     const idx_type n_chunks   = (N + chunk_size - 1) / chunk_size;
     idx_type nteams           = n_chunks > 512 ? 512 : n_chunks;
@@ -76,14 +65,14 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
           a_functor_reducer.get_reducer();
 #pragma omp parallel num_threads(team_size)
       {
-        const idx_type local_offset = team_id * chunk_size;
+        const idx_type local_offset = team_id * chunk_size + begin;
 
 #pragma omp for
         for (idx_type i = 0; i < chunk_size; ++i) {
           const idx_type idx = local_offset + i;
           value_type val;
           reducer.init(&val);
-          if (idx < N) a_functor(idx, val, false);
+          if ((idx >= begin) && (idx < end)) a_functor(idx, val, false);
 
           element_values(team_id, i) = val;
         }
@@ -118,7 +107,7 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
           a_functor_reducer.get_reducer();
 #pragma omp parallel num_threads(team_size)
       {
-        const idx_type local_offset = team_id * chunk_size;
+        const idx_type local_offset = team_id * chunk_size + begin;
         value_type offset_value;
         if (team_id > 0)
           offset_value = chunk_values(team_id - 1);
@@ -132,8 +121,9 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
           if (i > 0) {
             local_offset_value = element_values(team_id, i - 1);
             // FIXME_OPENMPTARGET We seem to access memory illegaly on AMD GPUs
-#if defined(KOKKOS_ARCH_AMD_GPU) && !defined(KOKKOS_ARCH_AMD_GFX1030) && \
-    !defined(KOKKOS_ARCH_AMD_GFX1100) && !defined(KOKKOS_ARCH_AMD_GFX1103)
+#if defined(KOKKOS_ARCH_AMD_GPU) && !defined(KOKKOS_ARCH_AMD_GFX1030) &&      \
+    !defined(KOKKOS_ARCH_AMD_GFX1100) && !defined(KOKKOS_ARCH_AMD_GFX1103) && \
+    !defined(KOKKOS_ARCH_AMD_GFX1201)
             if constexpr (Analysis::Reducer::has_join_member_function()) {
               a_functor.get_functor().join(local_offset_value, offset_value);
             } else
@@ -143,9 +133,9 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
 #endif
           } else
             local_offset_value = offset_value;
-          if (idx < N) a_functor(idx, local_offset_value, true);
+          if (idx < end) a_functor(idx, local_offset_value, true);
 
-          if (idx == N - 1 && m_result_ptr_device_accessible)
+          if (idx == end - 1 && m_result_ptr_device_accessible)
             *m_result_ptr = local_offset_value;
         }
       }
@@ -155,8 +145,6 @@ class ParallelScan<FunctorType, Kokkos::RangePolicy<Traits...>,
   void execute() const {
     Experimental::Impl::OpenMPTargetInternal::verify_is_process(
         "Kokkos::Experimental::OpenMPTarget parallel_for");
-    Experimental::Impl::OpenMPTargetInternal::verify_initialized(
-        "Kokkos::Experimental::OpenMPTarget parallel_for");
     const idx_type N          = m_policy.end() - m_policy.begin();
     const idx_type chunk_size = 128;
     const idx_type n_chunks   = (N + chunk_size - 1) / chunk_size;
@@ -206,8 +194,6 @@ class ParallelScanWithTotal<FunctorType, Kokkos::RangePolicy<Traits...>,
   void execute() const {
     Experimental::Impl::OpenMPTargetInternal::verify_is_process(
         "Kokkos::Experimental::OpenMPTarget parallel_for");
-    Experimental::Impl::OpenMPTargetInternal::verify_initialized(
-        "Kokkos::Experimental::OpenMPTarget parallel_for");
     const int64_t N        = base_t::m_policy.end() - base_t::m_policy.begin();
     const int chunk_size   = 128;
     const int64_t n_chunks = (N + chunk_size - 1) / chunk_size;
