@@ -1,18 +1,5 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
 #ifndef KOKKOS_SYCL_PARALLEL_REDUCE_TEAM_HPP
 #define KOKKOS_SYCL_PARALLEL_REDUCE_TEAM_HPP
@@ -53,8 +40,8 @@ class Kokkos::Impl::ParallelReduce<CombinedFunctorReducerType,
   const Policy m_policy;
   const pointer_type m_result_ptr;
   const bool m_result_ptr_device_accessible;
-  size_type m_shmem_begin;
-  size_type m_shmem_size;
+  int m_shmem_begin;
+  int m_shmem_size;
   size_t m_scratch_size[2];
   const size_type m_league_size;
   int m_team_size;
@@ -198,8 +185,8 @@ class Kokkos::Impl::ParallelReduce<CombinedFunctorReducerType,
                                   ReducerType>) {
                   reference_type update =
                       reducer.init(&local_mem[local_id * value_count]);
-                  for (int league_rank = group_id; league_rank < league_size;
-                       league_rank += n_wgroups) {
+                  for (size_type league_rank = group_id;
+                       league_rank < league_size; league_rank += n_wgroups) {
                     const member_type team_member(
                         team_scratch_memory_L0
                             .get_multi_ptr<sycl::access::decorated::yes>(),
