@@ -1,22 +1,15 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
 #include <gtest/gtest.h>
 
+#include <Kokkos_Macros.hpp>
+#ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+import kokkos.core;
+import kokkos.core_impl;
+#else
 #include <Kokkos_Core.hpp>
+#endif
 
 #include <regex>
 #include <limits>
@@ -339,4 +332,29 @@ constexpr bool test_chunk_size_explicit() {
 
 static_assert(test_chunk_size_explicit());
 
+// The execution space is defaulted if not given to the constructor.
+TEST(TEST_CATEGORY, range_policy_default_space) {
+  using policy_t = Kokkos::RangePolicy<TEST_EXECSPACE>;
+
+  policy_t defaulted(42, 666);
+
+  ASSERT_EQ(defaulted.space(), TEST_EXECSPACE{});
+}
+
+// The execution space instance can be updated.
+TEST(TEST_CATEGORY, range_policy_impl_set_space) {
+  using policy_t = Kokkos::RangePolicy<TEST_EXECSPACE>;
+
+  const auto [exec_old, exec_new] =
+      Kokkos::Experimental::partition_space(TEST_EXECSPACE{}, 1, 1);
+
+  const policy_t policy_old(exec_old, 42, 666);
+  ASSERT_EQ(policy_old.space(), exec_old);
+
+  const policy_t policy_new(Kokkos::Impl::PolicyUpdate{}, policy_old, exec_new);
+  ASSERT_EQ(policy_new.space(), exec_new);
+  ASSERT_EQ(policy_new.begin(), 42);
+  ASSERT_EQ(policy_new.end(), 666);
+}
+
 }  // namespace
