@@ -1,20 +1,18 @@
-//@HEADER
-// ************************************************************************
-//
-//                        Kokkos v. 4.0
-//       Copyright (2022) National Technology & Engineering
-//               Solutions of Sandia, LLC (NTESS).
-//
-// Under the terms of Contract DE-NA0003525 with NTESS,
-// the U.S. Government retains certain rights in this software.
-//
-// Part of Kokkos, under the Apache License v2.0 with LLVM Exceptions.
-// See https://kokkos.org/LICENSE for license information.
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-//
-//@HEADER
+// SPDX-FileCopyrightText: Copyright Contributors to the Kokkos project
 
+#include <Kokkos_Macros.hpp>
+#ifdef KOKKOS_ENABLE_EXPERIMENTAL_CXX20_MODULES
+import kokkos.core;
+import kokkos.core_impl;
+#else
 #include <Kokkos_Core.hpp>
+#endif
+
+#include <desul/atomics.hpp>
+
+#include <cstddef>
+#include <type_traits>
 
 namespace {
 
@@ -55,13 +53,18 @@ constexpr bool test_view_typedefs_impl() {
   static_assert(std::is_same_v<typename ViewType::data_type, DataType>);
   static_assert(std::is_same_v<typename ViewType::const_data_type, typename data_analysis<DataType>::const_data_type>);
   static_assert(std::is_same_v<typename ViewType::non_const_data_type, typename data_analysis<DataType>::non_const_data_type>);
-  
+
+
+#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_5
   // FIXME: these should be deprecated and for proper testing (I.e. where this is different from data_type)
   // we would need ensemble types which use the hidden View dimension facility of View (i.e. which make
   // "specialize" not void)
+  KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_PUSH()
   static_assert(std::is_same_v<typename ViewType::scalar_array_type, DataType>);
   static_assert(std::is_same_v<typename ViewType::const_scalar_array_type, typename data_analysis<DataType>::const_data_type>);
   static_assert(std::is_same_v<typename ViewType::non_const_scalar_array_type, typename data_analysis<DataType>::non_const_data_type>);
+  KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_POP()
+#endif
 #ifdef KOKKOS_ENABLE_IMPL_VIEW_LEGACY
   static_assert(std::is_same_v<typename ViewType::specialize, void>);
 #endif
@@ -85,33 +88,38 @@ constexpr bool test_view_typedefs_impl() {
   static_assert(std::is_same_v<typename ViewType::memory_traits, MemoryTraitsType>);
   static_assert(std::is_same_v<typename ViewType::host_mirror_space, HostMirrorSpace>);
   static_assert(std::is_same_v<typename ViewType::size_type, typename ViewType::memory_space::size_type>);
- 
+
   // FIXME: should be deprecated in favor of reference
   static_assert(std::is_same_v<typename ViewType::reference_type, ReferenceType>);
   // FIXME: should be deprecated in favor of data_handle_type
   static_assert(std::is_same_v<typename ViewType::pointer_type, ValueType*>);
- 
+
   // =========================================
   // in Legacy View: some helper View variants
   // =========================================
   static_assert(std::is_same_v<typename ViewType::traits, ViewTraitsType>);
+#ifdef KOKKOS_ENABLE_DEPRECATED_CODE_5
+KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_PUSH()
   static_assert(std::is_same_v<typename ViewType::array_type,
                                Kokkos::View<typename ViewType::scalar_array_type, typename ViewType::array_layout,
-                                            typename ViewType::device_type, typename ViewTraitsType::hooks_policy,
+                                            typename ViewType::device_type,
+                                            typename ViewType::memory_traits>>);
+KOKKOS_IMPL_DISABLE_DEPRECATED_WARNINGS_POP()
+#endif
+  static_assert(std::is_same_v<typename ViewType::type,
+                               Kokkos::View<typename ViewType::data_type, typename ViewType::array_layout,
+                                            typename ViewType::device_type,
                                             typename ViewType::memory_traits>>);
   static_assert(std::is_same_v<typename ViewType::const_type,
                                Kokkos::View<typename ViewType::const_data_type, typename ViewType::array_layout,
-                                            typename ViewType::device_type, typename ViewTraitsType::hooks_policy,
-                                            typename ViewType::memory_traits>>);
+                                            typename ViewType::device_type, typename ViewType::memory_traits>>);
   static_assert(std::is_same_v<typename ViewType::non_const_type,
                                Kokkos::View<typename ViewType::non_const_data_type, typename ViewType::array_layout,
-                                            typename ViewType::device_type, typename ViewTraitsType::hooks_policy,
-                                            typename ViewType::memory_traits>>);
+                                            typename ViewType::device_type, typename ViewType::memory_traits>>);
   static_assert(std::is_same_v<typename ViewType::host_mirror_type,
                                Kokkos::View<typename ViewType::non_const_data_type, typename ViewType::array_layout,
                                             Kokkos::Device<Kokkos::DefaultHostExecutionSpace,
-                                                           typename ViewType::host_mirror_space::memory_space>,
-                                            typename ViewTraitsType::hooks_policy>>);
+                                                           typename ViewType::host_mirror_space::memory_space>>>);
 
   using uniform_layout_type = std::conditional_t<ViewType::rank()==0 || (ViewType::rank()==0 &&
                                                  std::is_same_v<Layout, Kokkos::LayoutRight>),
