@@ -91,10 +91,8 @@ DESUL_IMPL_ATOMIC_FETCH_OP_SHIFT_HOST_AND_DEVICE(rshift)
   template <class T, class MemoryOrder, class MemoryScope>                           \
   ANNOTATION void HOST_OR_DEVICE##_atomic_store(                                     \
       T* const dest, const T val, MemoryOrder order, MemoryScope scope) {            \
-    store_fetch_operator<T, const T>::apply(                                         \
-        HOST_OR_DEVICE##_atomic_fetch_oper(                                          \
-            store_fetch_operator<T, const T>(), dest, val, order, scope),            \
-        val);                                                                        \
+    (void)HOST_OR_DEVICE##_atomic_fetch_oper(                                        \
+        store_fetch_operator<T, const T>(), dest, val, order, scope);                \
   }
 
 DESUL_IMPL_ATOMIC_LOAD_AND_STORE(DESUL_IMPL_HOST_FUNCTION, host)
